/*
 * LyMapBehaviorInterface.java
 * Copyright (c) 2008, Lyria.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.data.*;
import leon.view.*;


/**
 * This interface defines the methods to implement in order to modify the behavior of map view
 * controller.
 *
 * @author  Lyria S.A.
 * @version $Id: LyMapBehaviorInterface.java,v 6.5 2007/12/05 09:01:02 jls Exp $
 * @see     LyMapController
 */
public interface LyMapBehaviorInterface extends LyPanelBehaviorInterface
{
	String __VERSION = "$Id: LyMapBehaviorInterface.java,v 6.5 2007/12/05 09:01:02 jls Exp $";

	/**
	 * Gets the application objects list (which are {@link leon.data.LyObject LyObject} instances),
	 * that must be represented as <u>node</u> objects.<br>
	 * <b>IMPORTANT:</b> if the returned list is a dynamic workspace, the mapController will notify
	 * for node updates in {@link #updateNode(LyObject, LyMapNode, LyMapController)}. Association
	 * between application object and graphical object is made in the {@link #buildNode(LyObject,
	 * LyMapController)} method.
	 *
	 * @param  controller current LyMapController instance
	 * @return the list of node objects, or null if the map does not have nodes
	 * @see    #buildNode(LyObject, LyMapController)
	 * @see    #updateNode(LyObject, LyMapNode, LyMapController)
	 */
	LyObjectList getNodeObjects(LyMapController controller);

	/**
	 * Gets the list of objects of the application (which are {@link leon.data.LyObject LyObject}
	 * instances), that must be represented as <u>link</u> objects.<br />
	 * <b>IMPORTANT:</b> if the returned list is a dynamic workspace, the mapController will notify
	 * for link updates in {@link #updateLink(LyObject, LyMapLink, LyMapController)}. Association
	 * between application object and graphical object is made in the {@link #buildLink(LyObject,
	 * LyMapController)} method.
	 *
	 * @param  controller current LyMapController instance
	 * @return the list of links objects, or null if the map does not have links
	 * @see    #buildLink(LyObject, LyMapController)
	 * @see    #updateLink(LyObject, LyMapLink, LyMapController)
	 */
	LyObjectList getLinkObjects(LyMapController controller);

	/**
	 * Gets the list of objects of the application (which are {@link leon.data.LyObject LyObject}
	 * instances), that must be represented as <u>region</u> objects.<br />
	 * <b>IMPORTANT:</b> if the returned list is a dynamic workspace the mapController will notify
	 * for region updates in {@link #updateRegion(LyObject, LyMapRegion, LyMapController)}.
	 * Association between application object and graphical object is made in {@link
	 * #buildRegion(LyObject, LyMapController)} method.
	 *
	 * @param  controller current LyMapController instance
	 * @return the list of region objects, or null if the map does not have regions
	 * @see    #buildRegion(LyObject, LyMapController)
	 * @see    #updateRegion(LyObject, LyMapRegion, LyMapController)
	 */
	LyObjectList getRegionObjects(LyMapController controller);

	/**
	 * Builds a node object instance from the given object.<br />
	 * <u>Note:</u> Node status (colors, borders, cross...) should be set in {@link
	 * #updateNode(LyObject, LyMapNode, LyMapController)}.
	 *
	 * @param  nodeObject application object, usually a LyObject instance but not mandatory
	 * @param  controller current LyMapController instance
	 * @return the built node view object.
	 */
	LyMapNode buildNode(LyObject nodeObject, LyMapController controller);

	/**
	 * Builds a link object instance from the application object.<br />
	 * <u>Note:</u> Link status (colors, icons, ...) should be set in {@link #updateLink(LyObject,
	 * LyMapLink, LyMapController)} .
	 *
	 * @param  linkObject application object, usually LyObject instance but not mandatory
	 * @param  controller current LyMapController instance
	 * @return the built link view object
	 */
	LyMapLink buildLink(LyObject linkObject, LyMapController controller);

	/**
	 * Builds a region object instance from the application object.<br />
	 * <u>Note:</u> Region status (colors, icons, ...) should be set in {@link
	 * #updateRegion(LyObject, LyMapRegion, LyMapController)} .
	 *
	 * @param  regionObject application object, usually LyObject instance but not mandatory
	 * @param  controller   current LyMapController instance
	 * @return the built region view object
	 */
	LyMapRegion buildRegion(LyObject regionObject, LyMapController controller);

	/**
	 * Updates a the given mapNode object status (colors, label, border, style, ...) with the given
	 * nodeObject properties. The controller invokes this method when the given nodeObject is
	 * modified.
	 *
	 * @param nodeObject application object (reference)
	 * @param mapNode    view objects (need update)
	 * @param controller current LyMapController instance
	 */
	void updateNode(LyObject nodeObject, LyMapNode mapNode, LyMapController controller);

	/**
	 * Updates a the given mapLink object status (colors, label, border, style, ...) with the given
	 * linkObject properties. The controller invokes this method when the given linkObject is
	 * modified.
	 *
	 * @param linkObject application object (reference)
	 * @param mapLink    view objects (need update)
	 * @param controller current LyMapController instance
	 */
	void updateLink(LyObject linkObject, LyMapLink mapLink, LyMapController controller);

	/**
	 * Updates a the given mapRegion object status (colors, label, border, style, ...) with the
	 * given regionObject properties. The controller invokes this method when the given regionObject
	 * is modified.
	 *
	 * @param regionObject application object (reference)
	 * @param mapRegion    view objects (need update)
	 * @param controller   current LyMapController instance
	 */
	void updateRegion(LyObject regionObject, LyMapRegion mapRegion,
		LyMapController controller);

	/**
	 * This method is invoked by the controller when the given mapNode is selected on the map.
	 *
	 * @param nodeObject application object (reference)
	 * @param mapNode    view objects
	 * @param controller current LyMapController instance
	 */
	void selectNode(LyObject nodeObject, LyMapNode mapNode, LyMapController controller);

	/**
	 * This method is invoked by the controller to invoke the default action when an object is
	 * activated on the map (e.g. by double click with graphical modality).<br />
	 *
	 * @param      nodeObject application object (may be null)
	 * @param      mapObject  view objects (selected for action)
	 * @param      controller current LyMapController instance
	 * @return     true if the action was handled, false otherwise
	 * @deprecated replaced by {@link #invokeDefaultAction(LyObject, LyComponent, int, int,
	 *             LyMapController)}
	 */
	boolean invokeDefaultAction(LyObject nodeObject, LyComponent mapObject,
		LyMapController controller);

	/**
	 * This method is invoked by the controller to invoke the default action when an object is
	 * activated on the map (e.g. by double click with graphical modality).<br />
	 *
	 * @param  nodeObject application object (may be null)
	 * @param  mapObject  view objects (selected for action)
	 * @param  x          the x position of the event on the map
	 * @param  y          the y position of the event on the map
	 * @param  controller current LyMapController instance
	 * @return true if the action was handled, false otherwise
	 */
	boolean invokeDefaultAction(LyObject nodeObject, LyComponent mapObject, int x, int y,
		LyMapController controller);

	/**
	 * This method is invoked by the controller when the given mapNode is unselected on the map.
	 *
	 * @param nodeObject application object (reference)
	 * @param mapNode    view objects
	 * @param controller current LyMapController instance
	 */
	void unselectNode(LyObject nodeObject, LyMapNode mapNode, LyMapController controller);

	/**
	 * This method is invoked by the controller when all nodes are unselected on the map.
	 *
	 * @param controller current LyMapController instance
	 */
	void unselectNodes(LyMapController controller);

	/**
	 * This method is invoked by the controller when selected nodes on the map have been moved.
	 * <br />
	 *
	 * @param controller current mapController instance
	 */
	void selectionMoved(LyMapController controller);

	/**
	 * This method is invoked when the view is ready to be shown.
	 *
	 * @param controller current mapController instance
	 */
	void completeMap(LyMapController controller);

	/**
	 * This method is invoked when the content of the map has been modified.
	 *
	 * @param controller current mapController instance
	 * @param map        current map instance
	 * @param content    new map content instance
	 */
	void setMapContent(LyMapController controller, LyMap map, LyMapContent content);

	/**
	 * Frees all resources.
	 *
	 * @param controller current mapController instance
	 */
	void free(LyMapController controller);

	/**
	 * Method invoked at the beginning of the given object drag.<br />
	 *
	 * @param controller   current LyMapController instance
	 * @param sourceObject dragged object
	 */
	void dragStart(LyMapController controller, LyObject sourceObject);

	/**
	 * Method called during a drag-and-drop, when the given object is dragged over the map
	 * component.
	 *
	 * @param controller   current LyMapController instance
	 * @param sourceObject the dragged object
	 * @param x            The x position of the drag event
	 * @param y            The y position of the drag event
	 */
	void dragOver(LyMapController controller, LyObject sourceObject, int x, int y);

	/**
	 * Method called when the given dragged object is dropped on the given dropped object.
	 *
	 * @param controller   current LyMapController instance
	 * @param targetObject object on where the dragged object is dropped
	 * @param sourceObject the dragged object
	 * @param x            The x position of drop event
	 * @param y            The y position of drop event
	 */
	void drop(LyMapController controller, LyObject targetObject, LyObject sourceObject,
		int x, int y);

	/**
	 * Method called at the end of a drag-and-drop.<br />
	 *
	 * @param controller current LyMapController instance
	 * @param type       drag-and-drop type ({@link org.eclipse.swt.dnd.DND#DROP_COPY} or {@link
	 *                   org.eclipse.swt.dnd.DND#DROP_MOVE})
	 */
	void dragFinished(LyMapController controller, int type);
}